@@ -24,7 +24,7 @@ use std::ops::Deref;
 
 /// An X.509 certificate, as used in rustls.
 /// Corresponds to `Certificate` in the Rust API.
-/// <https://docs.rs/rustls/0.20.0/rustls/struct.Certificate.html>
+/// <https://docs.rs/rustls/latest/rustls/struct.Certificate.html>
 pub struct rustls_certificate {
     // We use the opaque struct pattern to tell C about our types without
     // telling them what's inside.
@@ -253,7 +253,7 @@ mod tests {
 /// The complete chain of certificates to send during a TLS handshake,
 /// plus a private key that matches the end-entity (leaf) certificate.
 /// Corresponds to `CertifiedKey` in the Rust API.
-/// <https://docs.rs/rustls/0.20.0/rustls/sign/struct.CertifiedKey.html>
+/// <https://docs.rs/rustls/latest/rustls/sign/struct.CertifiedKey.html>
 pub struct rustls_certified_key {
     // We use the opaque struct pattern to tell C about our types without
     // telling them what's inside.
@@ -379,13 +379,7 @@ impl rustls_certified_key {
     #[no_mangle]
     pub extern "C" fn rustls_certified_key_free(key: *const rustls_certified_key) {
         ffi_panic_boundary! {
-            if key.is_null() {
-                return;
-            }
-            // To free the certified_key, we reconstruct the Arc. It should have a refcount of 1,
-            // representing the C code's copy. When it drops, that refcount will go down to 0
-            // and the inner ServerConfig will be dropped.
-            unsafe { drop(Arc::from_raw(key)) };
+            rustls_certified_key::free(key);
         }
     }
 
@@ -399,13 +393,13 @@ impl rustls_certified_key {
             if cert_chain.is_null() {
                 return Err(NullParameter);
             }
-            slice::from_raw_parts(cert_chain, cert_chain_len as usize)
+            slice::from_raw_parts(cert_chain, cert_chain_len)
         };
         let private_key: &[u8] = unsafe {
             if private_key.is_null() {
                 return Err(NullParameter);
             }
-            slice::from_raw_parts(private_key, private_key_len as usize)
+            slice::from_raw_parts(private_key, private_key_len)
         };
         let mut private_keys: Vec<Vec<u8>> = match pkcs8_private_keys(&mut Cursor::new(private_key))
         {
@@ -440,7 +434,7 @@ impl rustls_certified_key {
 }
 
 /// A root certificate store.
-/// <https://docs.rs/rustls/0.20.0/rustls/struct.RootCertStore.html>
+/// <https://docs.rs/rustls/latest/rustls/struct.RootCertStore.html>
 pub struct rustls_root_cert_store {
     // We use the opaque struct pattern to tell C about our types without
     // telling them what's inside.
@@ -458,7 +452,7 @@ impl rustls_root_cert_store {
     /// Create a rustls_root_cert_store. Caller owns the memory and must
     /// eventually call rustls_root_cert_store_free. The store starts out empty.
     /// Caller must add root certificates with rustls_root_cert_store_add_pem.
-    /// <https://docs.rs/rustls/0.20.0/rustls/struct.RootCertStore.html#method.empty>
+    /// <https://docs.rs/rustls/latest/rustls/struct.RootCertStore.html#method.empty>
     #[no_mangle]
     pub extern "C" fn rustls_root_cert_store_new() -> *mut rustls_root_cert_store {
         ffi_panic_boundary! {
@@ -540,8 +534,11 @@ impl rustls_client_cert_verifier {
     pub extern "C" fn rustls_client_cert_verifier_new(
         store: *const rustls_root_cert_store,
     ) -> *const rustls_client_cert_verifier {
-        let store: &RootCertStore = try_ref_from_ptr!(store);
-        return Arc::into_raw(AllowAnyAuthenticatedClient::new(store.clone())) as *const _;
+        ffi_panic_boundary! {
+            let store: &RootCertStore = try_ref_from_ptr!(store);
+            let client_cert_verifier = AllowAnyAuthenticatedClient::new(store.clone());
+            return Arc::into_raw(client_cert_verifier.boxed()) as *const _;
+        }
     }
 
     /// "Free" a verifier previously returned from
@@ -555,13 +552,7 @@ impl rustls_client_cert_verifier {
         verifier: *const rustls_client_cert_verifier,
     ) {
         ffi_panic_boundary! {
-            if verifier.is_null() {
-                return;
-            }
-            // To free the verifier, we reconstruct the Arc. It should have a refcount of 1,
-            // representing the C code's copy. When it drops, that refcount will go down to 0
-            // and the inner object will be dropped.
-            unsafe { drop(Arc::from_raw(verifier)) };
+            rustls_client_cert_verifier::free(verifier);
         }
     }
 }
@@ -594,9 +585,12 @@ impl rustls_client_cert_verifier_optional {
     pub extern "C" fn rustls_client_cert_verifier_optional_new(
         store: *const rustls_root_cert_store,
     ) -> *const rustls_client_cert_verifier_optional {
-        let store: &RootCertStore = try_ref_from_ptr!(store);
-        return Arc::into_raw(AllowAnyAnonymousOrAuthenticatedClient::new(store.clone()))
-            as *const _;
+        ffi_panic_boundary! {
+            let store: &RootCertStore = try_ref_from_ptr!(store);
+            let client_cert_verifier = AllowAnyAnonymousOrAuthenticatedClient::new(store.clone());
+            return Arc::into_raw(client_cert_verifier.boxed())
+                as *const _;
+        }
     }
 
     /// "Free" a verifier previously returned from
@@ -610,13 +604,7 @@ impl rustls_client_cert_verifier_optional {
         verifier: *const rustls_client_cert_verifier_optional,
     ) {
         ffi_panic_boundary! {
-            if verifier.is_null() {
-                return;
-            }
-            // To free the verifier, we reconstruct the Arc. It should have a refcount of 1,
-            // representing the C code's copy. When it drops, that refcount will go down to 0
-            // and the inner object will be dropped.
-            unsafe { drop(Arc::from_raw(verifier)) };
+            rustls_client_cert_verifier_optional::free(verifier);
         }
     }
 }
