@@ -1,3 +1,4 @@
+use std::borrow::Cow;
 use std::convert::{TryFrom, TryInto};
 use std::ffi::{CStr, OsStr};
 use std::fs::File;
@@ -9,8 +10,8 @@ use std::time::SystemTime;
 use libc::{c_char, size_t};
 use rustls::client::{ResolvesClientCert, ServerCertVerified, ServerCertVerifier};
 use rustls::{
-    sign::CertifiedKey, Certificate, ClientConfig, ClientConnection, ProtocolVersion,
-    RootCertStore, SupportedCipherSuite, WantsVerifier, ALL_CIPHER_SUITES,
+    sign::CertifiedKey, Certificate, CertificateError, ClientConfig, ClientConnection,
+    ProtocolVersion, RootCertStore, SupportedCipherSuite, WantsVerifier, ALL_CIPHER_SUITES,
 };
 
 use crate::cipher::{rustls_certified_key, rustls_root_cert_store, rustls_supported_ciphersuite};
@@ -29,8 +30,8 @@ use crate::{
 /// done configuring settings, call rustls_client_config_builder_build
 /// to turn it into a *rustls_client_config. This object is not safe
 /// for concurrent mutation. Under the hood, it corresponds to a
-/// Box<ClientConfig>.
-/// <https://docs.rs/rustls/0.20.0/rustls/struct.ConfigBuilder.html>
+/// `Box<ClientConfig>`.
+/// <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
 pub struct rustls_client_config_builder {
     // We use the opaque struct pattern to tell C about our types without
     // telling them what's inside.
@@ -53,8 +54,8 @@ impl CastPtr for rustls_client_config_builder {
 impl BoxCastPtr for rustls_client_config_builder {}
 
 /// A client config that is done being constructed and is now read-only.
-/// Under the hood, this object corresponds to an Arc<ClientConfig>.
-/// <https://docs.rs/rustls/0.20.0/rustls/struct.ClientConfig.html>
+/// Under the hood, this object corresponds to an `Arc<ClientConfig>`.
+/// <https://docs.rs/rustls/latest/rustls/struct.ClientConfig.html>
 pub struct rustls_client_config {
     // We use the opaque struct pattern to tell C about our types without
     // telling them what's inside.
@@ -80,7 +81,9 @@ impl ServerCertVerifier for NoneVerifier {
         _ocsp_response: &[u8],
         _now: SystemTime,
     ) -> Result<ServerCertVerified, rustls::Error> {
-        Err(rustls::Error::InvalidCertificateSignature)
+        Err(rustls::Error::InvalidCertificate(
+            CertificateError::BadSignature,
+        ))
     }
 }
 
@@ -174,12 +177,15 @@ impl rustls_client_config_builder {
 
 /// Input to a custom certificate verifier callback. See
 /// rustls_client_config_builder_dangerous_set_certificate_verifier().
+///
+/// server_name can contain a hostname, an IPv4 address in textual form, or an
+/// IPv6 address in textual form.
 #[allow(non_camel_case_types)]
 #[repr(C)]
 pub struct rustls_verify_server_cert_params<'a> {
     pub end_entity_cert_der: rustls_slice_bytes<'a>,
     pub intermediate_certs_der: &'a rustls_slice_slice_bytes<'a>,
-    pub dns_name: rustls_str<'a>,
+    pub server_name: rustls_str<'a>,
     pub ocsp_response: rustls_slice_bytes<'a>,
 }
 
@@ -231,11 +237,12 @@ impl rustls::client::ServerCertVerifier for Verifier {
         _now: SystemTime,
     ) -> Result<ServerCertVerified, rustls::Error> {
         let cb = self.callback;
-        let dns_name: &str = match server_name {
-            rustls::ServerName::DnsName(n) => n.as_ref(),
+        let server_name: Cow<'_, str> = match server_name {
+            rustls::ServerName::DnsName(n) => n.as_ref().into(),
+            rustls::ServerName::IpAddress(ip) => ip.to_string().into(),
             _ => return Err(rustls::Error::General("unknown name type".to_string())),
         };
-        let dns_name: rustls_str = match dns_name.try_into() {
+        let server_name: rustls_str = match server_name.as_ref().try_into() {
             Ok(r) => r,
             Err(NulByte {}) => return Err(rustls::Error::General("NUL byte in SNI".to_string())),
         };
@@ -243,13 +250,13 @@ impl rustls::client::ServerCertVerifier for Verifier {
         let intermediates: Vec<_> = intermediates.iter().map(|cert| cert.as_ref()).collect();
 
         let intermediates = rustls_slice_slice_bytes {
-            inner: &*intermediates,
+            inner: &intermediates,
         };
 
         let params = rustls_verify_server_cert_params {
             end_entity_cert_der: end_entity.as_ref().into(),
             intermediate_certs_der: &intermediates,
-            dns_name,
+            server_name,
             ocsp_response: ocsp_response.into(),
         };
         let userdata = userdata_get().map_err(|_| {
@@ -280,24 +287,16 @@ impl rustls_client_config_builder {
     /// to make such mutation safe.
     ///
     /// The callback receives certificate chain information as raw bytes.
-    /// Currently this library offers no functions for C code to parse the
-    /// certificates, so you'll need to bring your own certificate parsing library
+    /// Currently this library offers no functions to parse the certificates,
+    /// so you'll need to bring your own certificate parsing library
     /// if you need to parse them.
     ///
-    /// If you intend to write a verifier that accepts all certificates, be aware
-    /// that special measures are required for IP addresses. Rustls currently
-    /// (0.20.0) doesn't support building a ClientConnection with an IP address
-    /// (because it's not a valid DnsNameRef). One workaround is to detect IP
-    /// addresses and rewrite them to `example.invalid`, and _also_ to disable
-    /// SNI via rustls_client_config_builder_set_enable_sni (IP addresses don't
-    /// need SNI).
-    ///
     /// If the custom verifier accepts the certificate, it should return
     /// RUSTLS_RESULT_OK. Otherwise, it may return any other rustls_result error.
     /// Feel free to use an appropriate error from the RUSTLS_RESULT_CERT_*
     /// section.
     ///
-    /// <https://docs.rs/rustls/0.20.0/rustls/client/struct.DangerousClientConfig.html#method.set_certificate_verifier>
+    /// <https://docs.rs/rustls/latest/rustls/client/struct.DangerousClientConfig.html#method.set_certificate_verifier>
     #[no_mangle]
     pub extern "C" fn rustls_client_config_builder_dangerous_set_certificate_verifier(
         config_builder: *mut rustls_client_config_builder,
@@ -389,7 +388,7 @@ impl rustls_client_config_builder {
     /// This function makes a copy of the data in `protocols` and does not retain
     /// any pointers, so the caller can free the pointed-to memory after calling.
     ///
-    /// <https://docs.rs/rustls/0.20.0/rustls/client/struct.ClientConfig.html#structfield.alpn_protocols>
+    /// <https://docs.rs/rustls/latest/rustls/client/struct.ClientConfig.html#structfield.alpn_protocols>
     #[no_mangle]
     pub extern "C" fn rustls_client_config_builder_set_alpn_protocols(
         builder: *mut rustls_client_config_builder,
@@ -411,7 +410,7 @@ impl rustls_client_config_builder {
     }
 
     /// Enable or disable SNI.
-    /// <https://docs.rs/rustls/0.20.0/rustls/struct.ClientConfig.html#structfield.enable_sni>
+    /// <https://docs.rs/rustls/latest/rustls/struct.ClientConfig.html#structfield.enable_sni>
     #[no_mangle]
     pub extern "C" fn rustls_client_config_builder_set_enable_sni(
         config: *mut rustls_client_config_builder,
@@ -522,11 +521,7 @@ impl rustls_client_config {
     #[no_mangle]
     pub extern "C" fn rustls_client_config_free(config: *const rustls_client_config) {
         ffi_panic_boundary! {
-            let config: &ClientConfig = try_ref_from_ptr!(config);
-            // To free the rustls_client_config, we reconstruct the Arc and then drop it. It should
-            // have a refcount of 1, representing the C code's copy. When it drops, that
-            // refcount will go down to 0 and the inner ClientConfig will be dropped.
-            unsafe { drop(Arc::from_raw(config)) };
+            rustls_client_config::free(config);
         }
     }
 
@@ -536,32 +531,36 @@ impl rustls_client_config {
     /// non-error, the memory pointed to by `conn_out` is modified to point at a
     /// valid rustls_connection. The caller now owns the rustls_connection and must
     /// call `rustls_connection_free` when done with it.
+    ///
+    /// The server_name parameter can contain a hostname or an IP address in
+    /// textual form (IPv4 or IPv6). This function will return an error if it
+    /// cannot be parsed as one of those types.
     #[no_mangle]
     pub extern "C" fn rustls_client_connection_new(
         config: *const rustls_client_config,
-        hostname: *const c_char,
+        server_name: *const c_char,
         conn_out: *mut *mut rustls_connection,
     ) -> rustls_result {
         ffi_panic_boundary! {
-        let hostname: &CStr = unsafe {
-            if hostname.is_null() {
+        let server_name: &CStr = unsafe {
+            if server_name.is_null() {
                 return NullParameter;
             }
-            CStr::from_ptr(hostname)
+            CStr::from_ptr(server_name)
         };
         let config: Arc<ClientConfig> = try_arc_from_ptr!(config);
-        let hostname: &str = match hostname.to_str() {
+        let server_name: &str = match server_name.to_str() {
             Ok(s) => s,
             Err(std::str::Utf8Error { .. }) => return rustls_result::InvalidDnsNameError,
         };
-        let server_name: rustls::ServerName = match hostname.try_into() {
+        let server_name: rustls::ServerName = match server_name.try_into() {
             Ok(sn) => sn,
             Err(_) => return rustls_result::InvalidDnsNameError,
         };
         let client = ClientConnection::new(config, server_name).unwrap();
 
         // We've succeeded. Put the client on the heap, and transfer ownership
-        // to the caller. After this point, we must return CRUSTLS_OK so the
+        // to the caller. After this point, we must return rustls_result::Ok so the
         // caller knows it is responsible for this memory.
         let c = Connection::from_client(client);
         BoxCastPtr::set_mut_ptr(conn_out, c);
@@ -647,4 +646,21 @@ mod tests {
         );
         rustls_connection::rustls_connection_free(conn);
     }
+
+    #[test]
+    #[cfg_attr(miri, ignore)]
+    fn test_client_connection_new_ipaddress() {
+        let builder: *mut rustls_client_config_builder =
+            rustls_client_config_builder::rustls_client_config_builder_new();
+        let config = rustls_client_config_builder::rustls_client_config_builder_build(builder);
+        let mut conn: *mut rustls_connection = null_mut();
+        let result = rustls_client_config::rustls_client_connection_new(
+            config,
+            "198.51.100.198\0".as_ptr() as *const c_char,
+            &mut conn,
+        );
+        if !matches!(result, rustls_result::Ok) {
+            panic!("expected RUSTLS_RESULT_OK, got {:?}", result);
+        }
+    }
 }
