@@ -1,5 +1,5 @@
-#ifndef CRUSTLS_H
-#define CRUSTLS_H
+#ifndef RUSTLS_H
+#define RUSTLS_H
 
 #include <stdarg.h>
 #include <stdbool.h>
@@ -20,7 +20,8 @@ enum rustls_result {
   RUSTLS_RESULT_INVALID_PARAMETER = 7009,
   RUSTLS_RESULT_UNEXPECTED_EOF = 7010,
   RUSTLS_RESULT_PLAINTEXT_EMPTY = 7011,
-  RUSTLS_RESULT_CORRUPT_MESSAGE = 7100,
+  RUSTLS_RESULT_ACCEPTOR_NOT_READY = 7012,
+  RUSTLS_RESULT_ALREADY_USED = 7013,
   RUSTLS_RESULT_NO_CERTIFICATES_PRESENTED = 7101,
   RUSTLS_RESULT_DECRYPT_ERROR = 7102,
   RUSTLS_RESULT_FAILED_TO_GET_CURRENT_TIME = 7103,
@@ -31,15 +32,42 @@ enum rustls_result {
   RUSTLS_RESULT_BAD_MAX_FRAGMENT_SIZE = 7114,
   RUSTLS_RESULT_UNSUPPORTED_NAME_TYPE = 7115,
   RUSTLS_RESULT_ENCRYPT_ERROR = 7116,
-  RUSTLS_RESULT_CERT_INVALID_ENCODING = 7117,
-  RUSTLS_RESULT_CERT_INVALID_SIGNATURE_TYPE = 7118,
-  RUSTLS_RESULT_CERT_INVALID_SIGNATURE = 7119,
-  RUSTLS_RESULT_CERT_INVALID_DATA = 7120,
+  RUSTLS_RESULT_CERT_ENCODING_BAD = 7121,
+  RUSTLS_RESULT_CERT_EXPIRED = 7122,
+  RUSTLS_RESULT_CERT_NOT_YET_VALID = 7123,
+  RUSTLS_RESULT_CERT_REVOKED = 7124,
+  RUSTLS_RESULT_CERT_UNHANDLED_CRITICAL_EXTENSION = 7125,
+  RUSTLS_RESULT_CERT_UNKNOWN_ISSUER = 7126,
+  RUSTLS_RESULT_CERT_BAD_SIGNATURE = 7127,
+  RUSTLS_RESULT_CERT_NOT_VALID_FOR_NAME = 7128,
+  RUSTLS_RESULT_CERT_INVALID_PURPOSE = 7129,
+  RUSTLS_RESULT_CERT_APPLICATION_VERIFICATION_FAILURE = 7130,
+  RUSTLS_RESULT_CERT_OTHER_ERROR = 7131,
+  RUSTLS_RESULT_MESSAGE_HANDSHAKE_PAYLOAD_TOO_LARGE = 7133,
+  RUSTLS_RESULT_MESSAGE_INVALID_CCS = 7134,
+  RUSTLS_RESULT_MESSAGE_INVALID_CONTENT_TYPE = 7135,
+  RUSTLS_RESULT_MESSAGE_INVALID_CERT_STATUS_TYPE = 7136,
+  RUSTLS_RESULT_MESSAGE_INVALID_CERT_REQUEST = 7137,
+  RUSTLS_RESULT_MESSAGE_INVALID_DH_PARAMS = 7138,
+  RUSTLS_RESULT_MESSAGE_INVALID_EMPTY_PAYLOAD = 7139,
+  RUSTLS_RESULT_MESSAGE_INVALID_KEY_UPDATE = 7140,
+  RUSTLS_RESULT_MESSAGE_INVALID_SERVER_NAME = 7141,
+  RUSTLS_RESULT_MESSAGE_TOO_LARGE = 7142,
+  RUSTLS_RESULT_MESSAGE_TOO_SHORT = 7143,
+  RUSTLS_RESULT_MESSAGE_MISSING_DATA = 7144,
+  RUSTLS_RESULT_MESSAGE_MISSING_KEY_EXCHANGE = 7145,
+  RUSTLS_RESULT_MESSAGE_NO_SIGNATURE_SCHEMES = 7146,
+  RUSTLS_RESULT_MESSAGE_TRAILING_DATA = 7147,
+  RUSTLS_RESULT_MESSAGE_UNEXPECTED_MESSAGE = 7148,
+  RUSTLS_RESULT_MESSAGE_UNKNOWN_PROTOCOL_VERSION = 7149,
+  RUSTLS_RESULT_MESSAGE_UNSUPPORTED_COMPRESSION = 7150,
+  RUSTLS_RESULT_MESSAGE_UNSUPPORTED_CURVE_TYPE = 7151,
+  RUSTLS_RESULT_MESSAGE_UNSUPPORTED_KEY_EXCHANGE_ALGORITHM = 7152,
+  RUSTLS_RESULT_MESSAGE_INVALID_OTHER = 7153,
   RUSTLS_RESULT_PEER_INCOMPATIBLE_ERROR = 7107,
   RUSTLS_RESULT_PEER_MISBEHAVED_ERROR = 7108,
   RUSTLS_RESULT_INAPPROPRIATE_MESSAGE = 7109,
   RUSTLS_RESULT_INAPPROPRIATE_HANDSHAKE_MESSAGE = 7110,
-  RUSTLS_RESULT_CORRUPT_MESSAGE_PAYLOAD = 7111,
   RUSTLS_RESULT_GENERAL = 7112,
   RUSTLS_RESULT_ALERT_CLOSE_NOTIFY = 7200,
   RUSTLS_RESULT_ALERT_UNEXPECTED_MESSAGE = 7201,
@@ -96,10 +124,44 @@ typedef enum rustls_tls_version {
   RUSTLS_TLS_VERSION_TLSV1_3 = 772,
 } rustls_tls_version;
 
+/**
+ * A parsed ClientHello produced by a rustls_acceptor. It is used to check
+ * server name indication (SNI), ALPN protocols, signature schemes, and
+ * cipher suites. It can be combined with a rustls_server_config to build a
+ * rustls_connection.
+ */
+typedef struct rustls_accepted rustls_accepted;
+
+/**
+ * A buffer and parser for ClientHello bytes. This allows reading ClientHello
+ * before choosing a rustls_server_config. It's useful when the server
+ * config will be based on parameters in the ClientHello: server name
+ * indication (SNI), ALPN protocols, signature schemes, and cipher suites. In
+ * particular, if a server wants to do some potentially expensive work to load a
+ * certificate for a given hostname, rustls_acceptor allows doing that asynchronously,
+ * as opposed to rustls_server_config_builder_set_hello_callback(), which doesn't
+ * work well for asynchronous I/O.
+ *
+ * The general flow is:
+ *  - rustls_acceptor_new()
+ *  - Loop:
+ *    - Read bytes from the network it with rustls_acceptor_read_tls().
+ *    - If successful, parse those bytes with rustls_acceptor_accept().
+ *    - If that returns RUSTLS_RESULT_ACCEPTOR_NOT_READY, continue.
+ *    - Otherwise, break.
+ *  - If rustls_acceptor_accept() returned RUSTLS_RESULT_OK:
+ *    - Examine the resulting rustls_accepted.
+ *    - Create or select a rustls_server_config.
+ *    - Call rustls_accepted_into_connection().
+ *  - Otherwise, there was a problem with the ClientHello data and the
+ *    connection should be rejected.
+ */
+typedef struct rustls_acceptor rustls_acceptor;
+
 /**
  * An X.509 certificate, as used in rustls.
  * Corresponds to `Certificate` in the Rust API.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.Certificate.html>
+ * <https://docs.rs/rustls/latest/rustls/struct.Certificate.html>
  */
 typedef struct rustls_certificate rustls_certificate;
 
@@ -107,7 +169,7 @@ typedef struct rustls_certificate rustls_certificate;
  * The complete chain of certificates to send during a TLS handshake,
  * plus a private key that matches the end-entity (leaf) certificate.
  * Corresponds to `CertifiedKey` in the Rust API.
- * <https://docs.rs/rustls/0.20.0/rustls/sign/struct.CertifiedKey.html>
+ * <https://docs.rs/rustls/latest/rustls/sign/struct.CertifiedKey.html>
  */
 typedef struct rustls_certified_key rustls_certified_key;
 
@@ -132,8 +194,8 @@ typedef struct rustls_client_cert_verifier_optional rustls_client_cert_verifier_
 
 /**
  * A client config that is done being constructed and is now read-only.
- * Under the hood, this object corresponds to an Arc<ClientConfig>.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.ClientConfig.html>
+ * Under the hood, this object corresponds to an `Arc<ClientConfig>`.
+ * <https://docs.rs/rustls/latest/rustls/struct.ClientConfig.html>
  */
 typedef struct rustls_client_config rustls_client_config;
 
@@ -143,8 +205,8 @@ typedef struct rustls_client_config rustls_client_config;
  * done configuring settings, call rustls_client_config_builder_build
  * to turn it into a *rustls_client_config. This object is not safe
  * for concurrent mutation. Under the hood, it corresponds to a
- * Box<ClientConfig>.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.ConfigBuilder.html>
+ * `Box<ClientConfig>`.
+ * <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
  */
 typedef struct rustls_client_config_builder rustls_client_config_builder;
 
@@ -160,14 +222,14 @@ typedef struct rustls_iovec rustls_iovec;
 
 /**
  * A root certificate store.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.RootCertStore.html>
+ * <https://docs.rs/rustls/latest/rustls/struct.RootCertStore.html>
  */
 typedef struct rustls_root_cert_store rustls_root_cert_store;
 
 /**
  * A server config that is done being constructed and is now read-only.
- * Under the hood, this object corresponds to an Arc<ServerConfig>.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.ServerConfig.html>
+ * Under the hood, this object corresponds to an `Arc<ServerConfig>`.
+ * <https://docs.rs/rustls/latest/rustls/struct.ServerConfig.html>
  */
 typedef struct rustls_server_config rustls_server_config;
 
@@ -177,7 +239,7 @@ typedef struct rustls_server_config rustls_server_config;
  * done configuring settings, call rustls_server_config_builder_build
  * to turn it into a *const rustls_server_config. This object is not safe
  * for concurrent mutation.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.ConfigBuilder.html>
+ * <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
  */
 typedef struct rustls_server_config_builder rustls_server_config_builder;
 
@@ -240,6 +302,30 @@ typedef struct rustls_str {
   size_t len;
 } rustls_str;
 
+/**
+ * A return value for a function that may return either success (0) or a
+ * non-zero value representing an error. The values should match socket
+ * error numbers for your operating system - for example, the integers for
+ * ETIMEDOUT, EAGAIN, or similar.
+ */
+typedef int rustls_io_result;
+
+/**
+ * A callback for rustls_connection_read_tls.
+ * An implementation of this callback should attempt to read up to n bytes from the
+ * network, storing them in `buf`. If any bytes were stored, the implementation should
+ * set out_n to the number of bytes stored and return 0. If there was an error,
+ * the implementation should return a nonzero rustls_io_result, which will be
+ * passed through to the caller. On POSIX systems, returning `errno` is convenient.
+ * On other systems, any appropriate error code works.
+ * It's best to make one read attempt to the network per call. Additional reads will
+ * be triggered by subsequent calls to one of the `_read_tls` methods.
+ * `userdata` is set to the value provided to `rustls_connection_set_userdata`. In most
+ * cases that should be a struct that contains, at a minimum, a file descriptor.
+ * The buf and out_n pointers are borrowed and should not be retained across calls.
+ */
+typedef rustls_io_result (*rustls_read_callback)(void *userdata, uint8_t *buf, size_t n, size_t *out_n);
+
 /**
  * A read-only view on a Rust byte slice.
  *
@@ -267,11 +353,14 @@ typedef void *rustls_verify_server_cert_user_data;
 /**
  * Input to a custom certificate verifier callback. See
  * rustls_client_config_builder_dangerous_set_certificate_verifier().
+ *
+ * server_name can contain a hostname, an IPv4 address in textual form, or an
+ * IPv6 address in textual form.
  */
 typedef struct rustls_verify_server_cert_params {
   struct rustls_slice_bytes end_entity_cert_der;
   const struct rustls_slice_slice_bytes *intermediate_certs_der;
-  struct rustls_str dns_name;
+  struct rustls_str server_name;
   struct rustls_slice_bytes ocsp_response;
 } rustls_verify_server_cert_params;
 
@@ -286,30 +375,6 @@ typedef struct rustls_log_params {
 
 typedef void (*rustls_log_callback)(void *userdata, const struct rustls_log_params *params);
 
-/**
- * A return value for a function that may return either success (0) or a
- * non-zero value representing an error. The values should match socket
- * error numbers for your operating system - for example, the integers for
- * ETIMEDOUT, EAGAIN, or similar.
- */
-typedef int rustls_io_result;
-
-/**
- * A callback for rustls_connection_read_tls.
- * An implementation of this callback should attempt to read up to n bytes from the
- * network, storing them in `buf`. If any bytes were stored, the implementation should
- * set out_n to the number of bytes stored and return 0. If there was an error,
- * the implementation should return a nonzero rustls_io_result, which will be
- * passed through to the caller. On POSIX systems, returning `errno` is convenient.
- * On other systems, any appropriate error code works.
- * It's best to make one read attempt to the network per call. Additional reads will
- * be triggered by subsequent calls to one of the `_read_tls` methods.
- * `userdata` is set to the value provided to `rustls_connection_set_userdata`. In most
- * cases that should be a struct that contains, at a minimum, a file descriptor.
- * The buf and out_n pointers are borrowed and should not be retained across calls.
- */
-typedef rustls_io_result (*rustls_read_callback)(void *userdata, uint8_t *buf, size_t n, size_t *out_n);
-
 /**
  * A callback for rustls_connection_write_tls.
  * An implementation of this callback should attempt to write the `n` bytes in buf
@@ -367,11 +432,12 @@ typedef struct rustls_slice_u16 {
 
 /**
  * The TLS Client Hello information provided to a ClientHelloCallback function.
- * `sni_name` is the SNI servername provided by the client. If the client
- * did not provide an SNI, the length of this `rustls_string` will be 0. The
- * signature_schemes carries the values supplied by the client or, should
- * the client not use this TLS extension, the default schemes in the rustls
- * library. See: <https://docs.rs/rustls/0.20.0/rustls/internal/msgs/enums/enum.SignatureScheme.html>.
+ * `server_name` is the value of the ServerNameIndication extension provided
+ * by the client. If the client did not send an SNI, the length of this
+ * `rustls_string` will be 0. The signature_schemes field carries the values
+ * supplied by the client or, if the client did not send this TLS extension,
+ * the default schemes in the rustls library. See:
+ * <https://docs.rs/rustls/latest/rustls/internal/msgs/enums/enum.SignatureScheme.html>.
  * `alpn` carries the list of ALPN protocol names that the client proposed to
  * the server. Again, the length of this list will be 0 if none were supplied.
  *
@@ -383,7 +449,7 @@ typedef struct rustls_slice_u16 {
  * the rustls library is re-evaluating their current approach to client hello handling.
  */
 typedef struct rustls_client_hello {
-  struct rustls_str sni_name;
+  struct rustls_str server_name;
   struct rustls_slice_u16 signature_schemes;
   const struct rustls_slice_slice_bytes *alpn;
 } rustls_client_hello;
@@ -482,6 +548,241 @@ extern const size_t RUSTLS_DEFAULT_VERSIONS_LEN;
  */
 struct rustls_str rustls_version(void);
 
+/**
+ * Create and return a new rustls_acceptor.
+ *
+ * Caller owns the pointed-to memory and must eventually free it with
+ * `rustls_acceptor_free()`.
+ */
+struct rustls_acceptor *rustls_acceptor_new(void);
+
+/**
+ * Free a rustls_acceptor.
+ *
+ * Parameters:
+ *
+ * acceptor: The rustls_acceptor to free.
+ *
+ * Calling with NULL is fine. Must not be called twice with the same value.
+ */
+void rustls_acceptor_free(struct rustls_acceptor *acceptor);
+
+/**
+ * Read some TLS bytes from the network into internal buffers. The actual network
+ * I/O is performed by `callback`, which you provide. Rustls will invoke your
+ * callback with a suitable buffer to store the read bytes into. You don't have
+ * to fill it up, just fill with as many bytes as you get in one syscall.
+ *
+ * Parameters:
+ *
+ * acceptor: The rustls_acceptor to read bytes into.
+ * callback: A function that will perform the actual network I/O.
+ *   Must be valid to call with the given userdata parameter until
+ *   this function call returns.
+ * userdata: An opaque parameter to be passed directly to `callback`.
+ *   Note: this is distinct from the `userdata` parameter set with
+ *   `rustls_connection_set_userdata`.
+ * out_n: An output parameter. This will be passed through to `callback`,
+ *   which should use it to store the number of bytes written.
+ *
+ * Returns:
+ *
+ * - 0: Success. You should call `rustls_acceptor_accept()` next.
+ * - Any non-zero value: error.
+ *
+ * This function passes through return values from `callback`. Typically
+ * `callback` should return an errno value. See `rustls_read_callback()` for
+ * more details.
+ */
+rustls_io_result rustls_acceptor_read_tls(struct rustls_acceptor *acceptor,
+                                          rustls_read_callback callback,
+                                          void *userdata,
+                                          size_t *out_n);
+
+/**
+ * Parse all TLS bytes read so far.  If those bytes make up a ClientHello,
+ * create a rustls_accepted from them.
+ *
+ * Parameters:
+ *
+ * acceptor: The rustls_acceptor to access.
+ * out_accepted: An output parameter. The pointed-to pointer will be set
+ *   to a new rustls_accepted only when the function returns
+ *   RUSTLS_RESULT_OK. The memory is owned by the caller and must eventually
+ *   be freed.
+ *
+ * Returns:
+ *
+ * - RUSTLS_RESULT_OK: a ClientHello has successfully been parsed.
+ *   A pointer to a newly allocated rustls_accepted has been written to
+ *   *out_accepted.
+ * - RUSTLS_RESULT_ACCEPTOR_NOT_READY: a full ClientHello has not yet been read.
+ *   Read more TLS bytes to continue.
+ * - Any other rustls_result: the TLS bytes read so far cannot be parsed
+ *   as a ClientHello, and reading additional bytes won't help.
+ *
+ * Memory and lifetimes:
+ *
+ * After this method returns RUSTLS_RESULT_OK, `acceptor` is
+ * still allocated and valid. It needs to be freed regardless of success
+ * or failure of this function.
+ *
+ * Calling `rustls_acceptor_accept()` multiple times on the same
+ * `rustls_acceptor` is acceptable from a memory perspective but pointless
+ * from a protocol perspective.
+ */
+rustls_result rustls_acceptor_accept(struct rustls_acceptor *acceptor,
+                                     struct rustls_accepted **out_accepted);
+
+/**
+ * Get the server name indication (SNI) from the ClientHello.
+ *
+ * Parameters:
+ *
+ * accepted: The rustls_accepted to access.
+ *
+ * Returns:
+ *
+ * A rustls_str containing the SNI field.
+ *
+ * The returned value is valid until rustls_accepted_into_connection or
+ * rustls_accepted_free is called on the same `accepted`. It is not owned
+ * by the caller and does not need to be freed.
+ *
+ * This will be a zero-length rustls_str in these error cases:
+ *
+ *  - The SNI contains a NUL byte.
+ *  - The `accepted` parameter was NULL.
+ *  - The `accepted` parameter was already transformed into a connection
+ *      with rustls_accepted_into_connection.
+ */
+struct rustls_str rustls_accepted_server_name(const struct rustls_accepted *accepted);
+
+/**
+ * Get the i'th in the list of signature schemes offered in the ClientHello.
+ * This is useful in selecting a server certificate when there are multiple
+ * available for the same server name, for instance when selecting
+ * between an RSA and an ECDSA certificate.
+ *
+ * Parameters:
+ *
+ * accepted: The rustls_accepted to access.
+ * i: Fetch the signature scheme at this offset.
+ *
+ * Returns:
+ *
+ * A TLS Signature Scheme from <https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-signaturescheme>
+ *
+ * This will be 0 in these cases:
+ *   - i is greater than the number of available cipher suites.
+ *   - accepted is NULL.
+ *   - rustls_accepted_into_connection has already been called with `accepted`.
+ */
+uint16_t rustls_accepted_signature_scheme(const struct rustls_accepted *accepted,
+                                          size_t i);
+
+/**
+ * Get the i'th in the list of cipher suites offered in the ClientHello.
+ *
+ * Parameters:
+ *
+ * accepted: The rustls_accepted to access.
+ * i: Fetch the cipher suite at this offset.
+ *
+ * Returns:
+ *
+ * A cipher suite value from <https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-4.>
+ *
+ * This will be 0 in these cases:
+ *   - i is greater than the number of available cipher suites.
+ *   - accepted is NULL.
+ *   - rustls_accepted_into_connection has already been called with `accepted`.
+ *
+ * Note that 0 is technically a valid cipher suite "TLS_NULL_WITH_NULL_NULL",
+ * but this library will never support null ciphers.
+ */
+uint16_t rustls_accepted_cipher_suite(const struct rustls_accepted *accepted,
+                                      size_t i);
+
+/**
+ * Get the i'th in the list of ALPN protocols requested in the ClientHello.
+ *
+ * accepted: The rustls_accepted to access.
+ * i: Fetch the ALPN value at this offset.
+ *
+ * Returns:
+ *
+ * A rustls_slice_bytes containing the i'th ALPN protocol. This may
+ * contain internal NUL bytes and is not guaranteed to contain valid
+ * UTF-8.
+ *
+ * This will be a zero-length rustls_slice bytes in these cases:
+ *   - i is greater than the number of offered ALPN protocols.
+ *   - The client did not offer the ALPN extension.
+ *   - The `accepted` parameter was already transformed into a connection
+ *      with rustls_accepted_into_connection.
+ *
+ * The returned value is valid until rustls_accepted_into_connection or
+ * rustls_accepted_free is called on the same `accepted`. It is not owned
+ * by the caller and does not need to be freed.
+ *
+ * If you are calling this from Rust, note that the `'static` lifetime
+ * in the return signature is fake and must not be relied upon.
+ */
+struct rustls_slice_bytes rustls_accepted_alpn(const struct rustls_accepted *accepted, size_t i);
+
+/**
+ * Turn a rustls_accepted into a rustls_connection, given the provided
+ * rustls_server_config.
+ *
+ * Parameters:
+ *
+ * accepted: The rustls_accepted to transform.
+ * config: The configuration with which to create this connection.
+ * out_conn: An output parameter. The pointed-to pointer will be set
+ *   to a new rustls_connection only when the function returns
+ *   RUSTLS_RESULT_OK.
+ *
+ * Returns:
+ *
+ * - RUSTLS_RESULT_OK: The `accepted` parameter was successfully
+ *   transformed into a rustls_connection, and *out_conn was written to.
+ * - RUSTLS_RESULT_ALREADY_USED: This function was called twice on the
+ *   same rustls_connection.
+ * - RUSTLS_RESULT_NULL_PARAMETER: One of the input parameters was NULL.
+ *
+ * Memory and lifetimes:
+ *
+ * In both success and failure cases, this consumes the contents of
+ * `accepted` but does not free its allocated memory. In either case,
+ * call rustls_accepted_free to avoid a memory leak.
+ *
+ * Calling accessor methods on an `accepted` after consuming it will
+ * return zero or default values.
+ *
+ * The rustls_connection emitted by this function in the success case
+ * is owned by the caller and must eventually be freed.
+ *
+ * This function does not take ownership of `config`. It does increment
+ * `config`'s internal reference count, indicating that the
+ * rustls_connection may hold a reference to it until it is done.
+ * See the documentation for rustls_connection for details.
+ */
+rustls_result rustls_accepted_into_connection(struct rustls_accepted *accepted,
+                                              const struct rustls_server_config *config,
+                                              struct rustls_connection **out_conn);
+
+/**
+ * Free a rustls_accepted.
+ *
+ * Parameters:
+ *
+ * accepted: The rustls_accepted to free.
+ *
+ * Calling with NULL is fine. Must not be called twice with the same value.
+ */
+void rustls_accepted_free(struct rustls_accepted *accepted);
+
 /**
  * Get the DER data of the certificate itself.
  * The data is owned by the certificate and has the same lifetime.
@@ -600,7 +901,7 @@ void rustls_certified_key_free(const struct rustls_certified_key *key);
  * Create a rustls_root_cert_store. Caller owns the memory and must
  * eventually call rustls_root_cert_store_free. The store starts out empty.
  * Caller must add root certificates with rustls_root_cert_store_add_pem.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.RootCertStore.html#method.empty>
+ * <https://docs.rs/rustls/latest/rustls/struct.RootCertStore.html#method.empty>
  */
 struct rustls_root_cert_store *rustls_root_cert_store_new(void);
 
@@ -716,24 +1017,16 @@ rustls_result rustls_client_config_builder_new_custom(const struct rustls_suppor
  * to make such mutation safe.
  *
  * The callback receives certificate chain information as raw bytes.
- * Currently this library offers no functions for C code to parse the
- * certificates, so you'll need to bring your own certificate parsing library
+ * Currently this library offers no functions to parse the certificates,
+ * so you'll need to bring your own certificate parsing library
  * if you need to parse them.
  *
- * If you intend to write a verifier that accepts all certificates, be aware
- * that special measures are required for IP addresses. Rustls currently
- * (0.20.0) doesn't support building a ClientConnection with an IP address
- * (because it's not a valid DnsNameRef). One workaround is to detect IP
- * addresses and rewrite them to `example.invalid`, and _also_ to disable
- * SNI via rustls_client_config_builder_set_enable_sni (IP addresses don't
- * need SNI).
- *
  * If the custom verifier accepts the certificate, it should return
  * RUSTLS_RESULT_OK. Otherwise, it may return any other rustls_result error.
  * Feel free to use an appropriate error from the RUSTLS_RESULT_CERT_*
  * section.
  *
- * <https://docs.rs/rustls/0.20.0/rustls/client/struct.DangerousClientConfig.html#method.set_certificate_verifier>
+ * <https://docs.rs/rustls/latest/rustls/client/struct.DangerousClientConfig.html#method.set_certificate_verifier>
  */
 rustls_result rustls_client_config_builder_dangerous_set_certificate_verifier(struct rustls_client_config_builder *config_builder,
                                                                               rustls_verify_server_cert_callback callback);
@@ -767,7 +1060,7 @@ rustls_result rustls_client_config_builder_load_roots_from_file(struct rustls_cl
  * This function makes a copy of the data in `protocols` and does not retain
  * any pointers, so the caller can free the pointed-to memory after calling.
  *
- * <https://docs.rs/rustls/0.20.0/rustls/client/struct.ClientConfig.html#structfield.alpn_protocols>
+ * <https://docs.rs/rustls/latest/rustls/client/struct.ClientConfig.html#structfield.alpn_protocols>
  */
 rustls_result rustls_client_config_builder_set_alpn_protocols(struct rustls_client_config_builder *builder,
                                                               const struct rustls_slice_bytes *protocols,
@@ -775,7 +1068,7 @@ rustls_result rustls_client_config_builder_set_alpn_protocols(struct rustls_clie
 
 /**
  * Enable or disable SNI.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.ClientConfig.html#structfield.enable_sni>
+ * <https://docs.rs/rustls/latest/rustls/struct.ClientConfig.html#structfield.enable_sni>
  */
 void rustls_client_config_builder_set_enable_sni(struct rustls_client_config_builder *config,
                                                  bool enable);
@@ -830,9 +1123,13 @@ void rustls_client_config_free(const struct rustls_client_config *config);
  * non-error, the memory pointed to by `conn_out` is modified to point at a
  * valid rustls_connection. The caller now owns the rustls_connection and must
  * call `rustls_connection_free` when done with it.
+ *
+ * The server_name parameter can contain a hostname or an IP address in
+ * textual form (IPv4 or IPv6). This function will return an error if it
+ * cannot be parsed as one of those types.
  */
 rustls_result rustls_client_connection_new(const struct rustls_client_config *config,
-                                           const char *hostname,
+                                           const char *server_name,
                                            struct rustls_connection **conn_out);
 
 /**
@@ -859,7 +1156,7 @@ void rustls_connection_set_log_callback(struct rustls_connection *conn, rustls_l
  * `rustls_connection_set_userdata`.
  * Returns 0 for success, or an errno value on error. Passes through return values
  * from callback. See rustls_read_callback for more details.
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.read_tls>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.read_tls>
  */
 rustls_io_result rustls_connection_read_tls(struct rustls_connection *conn,
                                             rustls_read_callback callback,
@@ -876,7 +1173,7 @@ rustls_io_result rustls_connection_read_tls(struct rustls_connection *conn,
  * `rustls_connection_set_userdata`.
  * Returns 0 for success, or an errno value on error. Passes through return values
  * from callback. See rustls_write_callback for more details.
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.write_tls>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.write_tls>
  */
 rustls_io_result rustls_connection_write_tls(struct rustls_connection *conn,
                                              rustls_write_callback callback,
@@ -893,7 +1190,7 @@ rustls_io_result rustls_connection_write_tls(struct rustls_connection *conn,
  * `rustls_connection_set_userdata`.
  * Returns 0 for success, or an errno value on error. Passes through return values
  * from callback. See rustls_write_callback for more details.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.Writer.html#method.write_vectored>
+ * <https://docs.rs/rustls/latest/rustls/struct.Writer.html#method.write_vectored>
  */
 rustls_io_result rustls_connection_write_tls_vectored(struct rustls_connection *conn,
                                                       rustls_write_vectored_callback callback,
@@ -904,22 +1201,22 @@ rustls_io_result rustls_connection_write_tls_vectored(struct rustls_connection *
  * Decrypt any available ciphertext from the internal buffer and put it
  * into the internal plaintext buffer, potentially making bytes available
  * for rustls_connection_read().
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.process_new_packets>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.process_new_packets>
  */
 rustls_result rustls_connection_process_new_packets(struct rustls_connection *conn);
 
 /**
- * <https://docs.rs/rustls/0.20.0/rustls/struct.CommonState.html#method.wants_read>
+ * <https://docs.rs/rustls/latest/rustls/struct.CommonState.html#method.wants_read>
  */
 bool rustls_connection_wants_read(const struct rustls_connection *conn);
 
 /**
- * <https://docs.rs/rustls/0.20.0/rustls/struct.CommonState.html#method.wants_write>
+ * <https://docs.rs/rustls/latest/rustls/struct.CommonState.html#method.wants_write>
  */
 bool rustls_connection_wants_write(const struct rustls_connection *conn);
 
 /**
- * <https://docs.rs/rustls/0.20.0/rustls/struct.CommonState.html#method.is_handshaking>
+ * <https://docs.rs/rustls/latest/rustls/struct.CommonState.html#method.is_handshaking>
  */
 bool rustls_connection_is_handshaking(const struct rustls_connection *conn);
 
@@ -928,13 +1225,13 @@ bool rustls_connection_is_handshaking(const struct rustls_connection *conn);
  * to completing the TLS handshake) and unsent TLS records. By default, there
  * is no limit. The limit can be set at any time, even if the current buffer
  * use is higher.
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.set_buffer_limit>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.set_buffer_limit>
  */
 void rustls_connection_set_buffer_limit(struct rustls_connection *conn, size_t n);
 
 /**
  * Queues a close_notify fatal alert to be sent in the next write_tls call.
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.send_close_notify>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.send_close_notify>
  */
 void rustls_connection_send_close_notify(struct rustls_connection *conn);
 
@@ -947,7 +1244,7 @@ void rustls_connection_send_close_notify(struct rustls_connection *conn);
  * affecting the connection. A mutating function call is one where the
  * first argument has type `struct rustls_connection *` (as opposed to
  *  `const struct rustls_connection *`).
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.peer_certificates>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.peer_certificates>
  */
 const struct rustls_certificate *rustls_connection_get_peer_certificate(const struct rustls_connection *conn,
                                                                         size_t i);
@@ -963,7 +1260,7 @@ const struct rustls_certificate *rustls_connection_get_peer_certificate(const st
  * first argument has type `struct rustls_connection *` (as opposed to
  *  `const struct rustls_connection *`).
  * <https://www.iana.org/assignments/tls-parameters/>
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.alpn_protocol>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.alpn_protocol>
  */
 void rustls_connection_get_alpn_protocol(const struct rustls_connection *conn,
                                          const uint8_t **protocol_out,
@@ -973,8 +1270,8 @@ void rustls_connection_get_alpn_protocol(const struct rustls_connection *conn,
  * Return the TLS protocol version that has been negotiated. Before this
  * has been decided during the handshake, this will return 0. Otherwise,
  * the u16 version number as defined in the relevant RFC is returned.
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.protocol_version>
- * <https://docs.rs/rustls/0.20.0/rustls/internal/msgs/enums/enum.ProtocolVersion.html>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.protocol_version>
+ * <https://docs.rs/rustls/latest/rustls/internal/msgs/enums/enum.ProtocolVersion.html>
  */
 uint16_t rustls_connection_get_protocol_version(const struct rustls_connection *conn);
 
@@ -982,7 +1279,7 @@ uint16_t rustls_connection_get_protocol_version(const struct rustls_connection *
  * Retrieves the cipher suite agreed with the peer.
  * This returns NULL until the ciphersuite is agreed.
  * The returned pointer lives as long as the program.
- * <https://docs.rs/rustls/0.20.0/rustls/enum.Connection.html#method.negotiated_cipher_suite>
+ * <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.negotiated_cipher_suite>
  */
 const struct rustls_supported_ciphersuite *rustls_connection_get_negotiated_ciphersuite(const struct rustls_connection *conn);
 
@@ -992,7 +1289,7 @@ const struct rustls_supported_ciphersuite *rustls_connection_get_negotiated_ciph
  * `rustls_connection_write_tls`.
  * On success, store the number of bytes actually written in *out_n
  * (this may be less than `count`).
- * <https://docs.rs/rustls/0.20.0/rustls/struct.Writer.html#method.write>
+ * <https://docs.rs/rustls/latest/rustls/struct.Writer.html#method.write>
  */
 rustls_result rustls_connection_write(struct rustls_connection *conn,
                                       const uint8_t *buf,
@@ -1011,7 +1308,7 @@ rustls_result rustls_connection_write(struct rustls_connection *conn,
  * read from it, the memory in `buf` must be initialized before the call (for
  * Rust-internal reasons). Initializing a buffer once and then using it
  * multiple times without zeroizing before each call is fine.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.Reader.html#method.read>
+ * <https://docs.rs/rustls/latest/rustls/struct.Reader.html#method.read>
  */
 rustls_result rustls_connection_read(struct rustls_connection *conn,
                                      uint8_t *buf,
@@ -1149,7 +1446,7 @@ void rustls_server_config_builder_free(struct rustls_server_config_builder *conf
  * With `ignore` != 0, the server will ignore the client ordering of cipher
  * suites, aka preference, during handshake and respect its own ordering
  * as configured.
- * <https://docs.rs/rustls/0.20.0/rustls/struct.ServerConfig.html#structfield.ignore_client_order>
+ * <https://docs.rs/rustls/latest/rustls/struct.ServerConfig.html#structfield.ignore_client_order>
  */
 rustls_result rustls_server_config_builder_set_ignore_client_order(struct rustls_server_config_builder *builder,
                                                                    bool ignore);
@@ -1164,7 +1461,7 @@ rustls_result rustls_server_config_builder_set_ignore_client_order(struct rustls
  * This function makes a copy of the data in `protocols` and does not retain
  * any pointers, so the caller can free the pointed-to memory after calling.
  *
- * <https://docs.rs/rustls/0.20.0/rustls/server/struct.ServerConfig.html#structfield.alpn_protocols>
+ * <https://docs.rs/rustls/latest/rustls/server/struct.ServerConfig.html#structfield.alpn_protocols>
  */
 rustls_result rustls_server_config_builder_set_alpn_protocols(struct rustls_server_config_builder *builder,
                                                               const struct rustls_slice_bytes *protocols,
@@ -1216,18 +1513,18 @@ rustls_result rustls_server_connection_new(const struct rustls_server_config *co
                                            struct rustls_connection **conn_out);
 
 /**
- * Copy the SNI hostname to `buf` which can hold up  to `count` bytes,
- * and the length of that hostname in `out_n`. The string is stored in UTF-8
- * with no terminating NUL byte.
+ * Copy the server name from the server name indication (SNI) extension to `buf` which can
+ * hold up  to `count` bytes, and the length of that server name in `out_n`. The string is
+ * stored in UTF-8 with no terminating NUL byte.
  * Returns RUSTLS_RESULT_INSUFFICIENT_SIZE if the SNI hostname is longer than `count`.
  * Returns Ok with *out_n == 0 if there is no SNI hostname available on this connection
  * because it hasn't been processed yet, or because the client did not send SNI.
- * <https://docs.rs/rustls/0.20.0/rustls/server/struct.ServerConnection.html#method.sni_hostname>
+ * <https://docs.rs/rustls/latest/rustls/server/struct.ServerConnection.html#method.server_name>
  */
-rustls_result rustls_server_connection_get_sni_hostname(const struct rustls_connection *conn,
-                                                        uint8_t *buf,
-                                                        size_t count,
-                                                        size_t *out_n);
+rustls_result rustls_server_connection_get_server_name(const struct rustls_connection *conn,
+                                                       uint8_t *buf,
+                                                       size_t count,
+                                                       size_t *out_n);
 
 /**
  * Register a callback to be invoked when a connection created from this config
@@ -1282,4 +1579,4 @@ rustls_result rustls_server_config_builder_set_persistence(struct rustls_server_
                                                            rustls_session_store_get_callback get_cb,
                                                            rustls_session_store_put_callback put_cb);
 
-#endif /* CRUSTLS_H */
+#endif /* RUSTLS_H */
