@@ -37,7 +37,7 @@ use crate::{
 /// done configuring settings, call rustls_server_config_builder_build
 /// to turn it into a *const rustls_server_config. This object is not safe
 /// for concurrent mutation.
-/// <https://docs.rs/rustls/0.20.0/rustls/struct.ConfigBuilder.html>
+/// <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
 pub struct rustls_server_config_builder {
     // We use the opaque struct pattern to tell C about our types without
     // telling them what's inside.
@@ -61,8 +61,8 @@ impl CastPtr for rustls_server_config_builder {
 impl BoxCastPtr for rustls_server_config_builder {}
 
 /// A server config that is done being constructed and is now read-only.
-/// Under the hood, this object corresponds to an Arc<ServerConfig>.
-/// <https://docs.rs/rustls/0.20.0/rustls/struct.ServerConfig.html>
+/// Under the hood, this object corresponds to an `Arc<ServerConfig>`.
+/// <https://docs.rs/rustls/latest/rustls/struct.ServerConfig.html>
 pub struct rustls_server_config {
     // We use the opaque struct pattern to tell C about our types without
     // telling them what's inside.
@@ -86,7 +86,7 @@ impl rustls_server_config_builder {
         ffi_panic_boundary! {
             let builder = ServerConfigBuilder {
                            base: rustls::ServerConfig::builder().with_safe_defaults(),
-                           verifier: NoClientAuth::new(),
+                           verifier: NoClientAuth::boxed(),
                            cert_resolver: None,
                            session_storage: None,
                            alpn_protocols: vec![],
@@ -148,7 +148,7 @@ impl rustls_server_config_builder {
 
             let builder = ServerConfigBuilder {
                 base,
-                verifier: NoClientAuth::new(),
+                verifier: NoClientAuth::boxed(),
                 cert_resolver: None,
                 session_storage: None,
                 alpn_protocols: vec![],
@@ -207,7 +207,7 @@ impl rustls_server_config_builder {
     /// With `ignore` != 0, the server will ignore the client ordering of cipher
     /// suites, aka preference, during handshake and respect its own ordering
     /// as configured.
-    /// <https://docs.rs/rustls/0.20.0/rustls/struct.ServerConfig.html#structfield.ignore_client_order>
+    /// <https://docs.rs/rustls/latest/rustls/struct.ServerConfig.html#structfield.ignore_client_order>
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_set_ignore_client_order(
         builder: *mut rustls_server_config_builder,
@@ -229,7 +229,7 @@ impl rustls_server_config_builder {
     /// This function makes a copy of the data in `protocols` and does not retain
     /// any pointers, so the caller can free the pointed-to memory after calling.
     ///
-    /// <https://docs.rs/rustls/0.20.0/rustls/server/struct.ServerConfig.html#structfield.alpn_protocols>
+    /// <https://docs.rs/rustls/latest/rustls/server/struct.ServerConfig.html#structfield.alpn_protocols>
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_set_alpn_protocols(
         builder: *mut rustls_server_config_builder,
@@ -317,11 +317,7 @@ impl rustls_server_config {
     #[no_mangle]
     pub extern "C" fn rustls_server_config_free(config: *const rustls_server_config) {
         ffi_panic_boundary! {
-            let config: &ServerConfig = try_ref_from_ptr!(config);
-            // To free the rustls_server_config, we reconstruct the Arc. It should have a refcount of 1,
-            // representing the C code's copy. When it drops, that refcount will go down to 0
-            // and the inner ServerConfig will be dropped.
-            unsafe { drop(Arc::from_raw(config)) };
+            rustls_server_config::free(config);
         }
     }
 
@@ -344,7 +340,7 @@ impl rustls_server_config {
                 Err(e) => return map_error(e),
             };
             // We've succeeded. Put the server on the heap, and transfer ownership
-            // to the caller. After this point, we must return CRUSTLS_OK so the
+            // to the caller. After this point, we must return rustls_result::Ok so the
             // caller knows it is responsible for this memory.
             let c = Connection::from_server(server_connection);
             BoxCastPtr::set_mut_ptr(conn_out, c);
@@ -353,15 +349,15 @@ impl rustls_server_config {
     }
 }
 
-/// Copy the SNI hostname to `buf` which can hold up  to `count` bytes,
-/// and the length of that hostname in `out_n`. The string is stored in UTF-8
-/// with no terminating NUL byte.
+/// Copy the server name from the server name indication (SNI) extension to `buf` which can
+/// hold up  to `count` bytes, and the length of that server name in `out_n`. The string is
+/// stored in UTF-8 with no terminating NUL byte.
 /// Returns RUSTLS_RESULT_INSUFFICIENT_SIZE if the SNI hostname is longer than `count`.
 /// Returns Ok with *out_n == 0 if there is no SNI hostname available on this connection
 /// because it hasn't been processed yet, or because the client did not send SNI.
-/// <https://docs.rs/rustls/0.20.0/rustls/server/struct.ServerConnection.html#method.sni_hostname>
+/// <https://docs.rs/rustls/latest/rustls/server/struct.ServerConnection.html#method.server_name>
 #[no_mangle]
-pub extern "C" fn rustls_server_connection_get_sni_hostname(
+pub extern "C" fn rustls_server_connection_get_server_name(
     conn: *const rustls_connection,
     buf: *mut u8,
     count: size_t,
@@ -379,7 +375,7 @@ pub extern "C" fn rustls_server_connection_get_sni_hostname(
             Some(s) => s,
             _ => return rustls_result::InvalidParameter,
         };
-        let sni_hostname = match server_connection.sni_hostname() {
+        let sni_hostname = match server_connection.server_name() {
             Some(sni_hostname) => sni_hostname,
             None => {
                 unsafe {
@@ -434,11 +430,12 @@ impl ResolvesServerCert for ResolvesServerCertFromChoices {
 }
 
 /// The TLS Client Hello information provided to a ClientHelloCallback function.
-/// `sni_name` is the SNI servername provided by the client. If the client
-/// did not provide an SNI, the length of this `rustls_string` will be 0. The
-/// signature_schemes carries the values supplied by the client or, should
-/// the client not use this TLS extension, the default schemes in the rustls
-/// library. See: <https://docs.rs/rustls/0.20.0/rustls/internal/msgs/enums/enum.SignatureScheme.html>.
+/// `server_name` is the value of the ServerNameIndication extension provided
+/// by the client. If the client did not send an SNI, the length of this
+/// `rustls_string` will be 0. The signature_schemes field carries the values
+/// supplied by the client or, if the client did not send this TLS extension,
+/// the default schemes in the rustls library. See:
+/// <https://docs.rs/rustls/latest/rustls/internal/msgs/enums/enum.SignatureScheme.html>.
 /// `alpn` carries the list of ALPN protocol names that the client proposed to
 /// the server. Again, the length of this list will be 0 if none were supplied.
 ///
@@ -450,7 +447,7 @@ impl ResolvesServerCert for ResolvesServerCertFromChoices {
 /// the rustls library is re-evaluating their current approach to client hello handling.
 #[repr(C)]
 pub struct rustls_client_hello<'a> {
-    sni_name: rustls_str<'a>,
+    server_name: rustls_str<'a>,
     signature_schemes: rustls_slice_u16<'a>,
     alpn: *const rustls_slice_slice_bytes<'a>,
 }
@@ -506,13 +503,13 @@ impl ClientHelloResolver {
 
 impl ResolvesServerCert for ClientHelloResolver {
     fn resolve(&self, client_hello: ClientHello) -> Option<Arc<CertifiedKey>> {
-        let sni_name: &str = {
+        let server_name: &str = {
             match client_hello.server_name() {
                 Some(c) => c,
                 None => "",
             }
         };
-        let sni_name: rustls_str = match sni_name.try_into() {
+        let server_name: rustls_str = match server_name.try_into() {
             Ok(r) => r,
             Err(_) => return None,
         };
@@ -530,7 +527,7 @@ impl ResolvesServerCert for ClientHelloResolver {
         let alpn = rustls_slice_slice_bytes { inner: &alpn };
         let signature_schemes: rustls_slice_u16 = (&*mapped_sigs).into();
         let hello = rustls_client_hello {
-            sni_name,
+            server_name,
             signature_schemes,
             alpn: &alpn,
         };
